/**
 *
 *  This class can store a chess board location. It also
 *  includes methods for converting character representations
 *  of locations to integer representations
 *  @author Nikos Massios
 *  @author Matthijs Spaan
 *  @version $Id: BoardLocation.java,v 1.4 2002/04/19 16:03:17 mtjspaan Exp $
 */
public class BoardLocation{
  /** 
   * stores the row of the chess board location (range 0-7)
   */
    public int row;
  /** 
   * stores the column of the chess board location (range 0-7)
   */
    public int column;


  /**
   * Constructor. It uses its argument to initialise
   * class fields row and column. 
   * @see #row
   * @see #column
   * @param loc The chess board location represented as a string.
   *          For example, "e2"
   */
    public BoardLocation(String loc){
    	char[] locc;
	
        locc = loc.toCharArray();
	column = getCol(locc[0]);
	row = getRow(locc[1]);
    }

  /**
   * Constructor. It uses its arguments to initialise
   * class fields row and column. 
   * @see #row
   * @see #column
   * @param column The column of the chess board location.
   *          For example, "e" is mapped 4.
   * @param row The row of the chess board location.
   *          For example, "5" is mapped 4.
   */
    public BoardLocation(int column, int row){
	
	this.column = column;
	this.row = row;
    }


  /**
   * Converts a character representing a chess board row 
   * to an integer.
   * For example character '3' is converted to integer 2.
   * @param row The chess board row as a character (range '1'-'8').
   * @return The chess board row as an integer (range 0-7).          
   */
    public int getRow(char row){
	int index;
	
	if(row  == '1'){
	    index = 0;
      	}
	else if(row == '2'){
	    index = 1;
	}
	else if(row == '3'){
	    index = 2;
	}
	else if(row == '4'){
	    index = 3;
	}
	else if(row == '5'){
	    index = 4;
	}
	else if(row == '6'){
	    index = 5;
	}
	else if(row == '7'){
	    index = 6;
	}
	else if(row == '8'){
	    index = 7;
	}
	else {
	    System.err.println("row passed: " + row);
	    System.err.println("allowed to use: 1-8");
	    index = -1;
	}
	return(index);
    }


  /**
   * Converts a character representing a chess board column 
   * to an integer.
   * For example characters 'b' or 'B' are converted to 
   * integer 1.
   * @param column The chess board column as a character (range 'A'-'H'
      and 'a'-'h').
   * @return The chess board column as an integer (range 0-7).          
   */
    public int getCol(char column){
	int index;
	
	if((column  == 'A') ||
	   (column == 'a') ){
	    index = 0;
      	}
	else if((column == 'B') ||
		(column == 'b') ){
	    index = 1;
	}
	else if((column == 'C') ||
		(column == 'c') ){
	    index = 2;
	}
	else if((column == 'D') ||
		(column == 'd') ){
	    index = 3;
	}
	else if((column == 'E') ||
		(column == 'e') ){
	    index = 4;
	}
	else if((column == 'F') ||
		(column == 'f') ){
	    index = 5;
	}
	else if((column == 'G') ||
		(column == 'g') ){
	    index = 6;
	}
	else if((column == 'H') ||
		(column == 'h') ){
	    index = 7;
	}
	else {
	    System.err.println("column passed: " + column);
	    System.err.println("allowed to use: A-H");
	    index = -1;
	}
	return(index);
    }



}
